/*
 * Copyright (c) 2004 - 2007, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.postgresql;

import java.sql.SQLException;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.security.auth.Subject;
import javax.sql.XAConnection;

import org.postgresql.xa.PGXADataSource;
import org.tranql.connector.CredentialExtractor;
import org.tranql.connector.NoExceptionsAreFatalSorter;
import org.tranql.connector.jdbc.AbstractXADataSourceMCF;
import org.tranql.connector.jdbc.ManagedXAConnection;


/**
 * ManagedConnectionFactory that wraps a PostgreSQL EmbeddedXADataSource for use
 * in-VM.
 *
 * @version $Revision: 555 $ $Date: 2007-09-28 13:19:49 -0700 (Fri, 28 Sep 2007) $
 */


public class PGXAMCF extends AbstractXADataSourceMCF {

    private final PGXADataSource ds;

    private String password = "";

    /**
     * Default constructor for a PostgreSQL Embedded XA DataSource.
     */
    public PGXAMCF() {
        super(new PGXADataSource(), new NoExceptionsAreFatalSorter());
        ds = (PGXADataSource) xaDataSource;
    }


    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo connectionRequestInfo) throws ResourceException {
        CredentialExtractor credentialExtractor = new CredentialExtractor(subject, connectionRequestInfo, this);

        XAConnection sqlConnection = getPhysicalConnection(subject, credentialExtractor);
        try {
            ManagedXAConnection mxac = null;
            mxac = new ManagedXAConnection(this, sqlConnection, credentialExtractor, exceptionSorter);

            return mxac;
        } catch (SQLException e) {
            throw new ResourceAdapterInternalException("Could not set up ManagedXAConnection", e);
        }
    }

    /* (non-Javadoc)
    * @see org.tranql.connector.UserPasswordManagedConnectionFactory#getUserName()
    */
    public String getUserName() {
        return ds.getUser();
    }

    /**
     * Set the user name used establish the connection. This value is used if no
     * connection information is supplied by the application when attempting to
     * create a connection.
     *
     * @param user the new user name
     */
    public void setUserName(String user) {
        ds.setUser(user);
    }

    /**
     * Return the password credential used to establish the connection.
     *
     * @return the password credential used to establish the connection
     */

    public String getPassword() {
        return password;
    }

    /**
     * Set the user password credential establish the connection. This value is
     * used if no connection information is supplied by the application when
     * attempting to create a connection.
     *
     * @param pwd the pwd
     */
    public void setPassword(String pwd) {
        this.password = pwd;
        ds.setPassword(pwd);
    }

    /**
     * Returns the name of the database. This name is used as the database
     * portion of the connection URL.
     *
     * @return the database name
     */
    public String getDatabaseName() {
        return ds.getDatabaseName();
    }

    /**
     * Specifies the name of the database to connect to. This name is used as
     * the database portion of the connection URL.
     *
     * @param name the database name
     */
    public void setDatabaseName(String name) {
        ds.setDatabaseName(name);
    }

    /**
     * Returns the port number used to connect to the remote database system.
     *
     * @return the port number
     */
    public Integer getPortNumber() {
        return new Integer(ds.getPortNumber());
    }


    /**
     * Specifies the port number to connect to for a remote database server.
     *
     * @param portNumber the port number
     */
    public void setPortNumber(Integer portNumber) {
        ds.setPortNumber(portNumber.intValue());
    }

    /**
     * Gets the prepare threshold.
     *
     * @return the prepare threshold
     * @see org.postgresql.ds.common.BaseDataSource#getPrepareThreshold()
     */
    public Integer getPrepareThreshold() {
        return new Integer(ds.getPrepareThreshold());
    }

    /**
     * Sets the prepare threshold.
     *
     * @param arg0 the arg0
     * @see org.postgresql.ds.common.BaseDataSource#setPrepareThreshold(int)
     */
    public void setPrepareThreshold(Integer arg0) {
        ds.setPrepareThreshold(arg0.intValue());
    }

    /**
     * Returns the name of the remote server providing RDBMS connectivity.
     *
     * @return the host name of the remote database server.
     */

    public String getServerName() {
        return ds.getServerName();
    }


    /**
     * Used to specify the name of the server that provides database services.
     *
     * @param serverName the server name
     */
    public void setServerName(String serverName) {
        ds.setServerName(serverName);
    }

}
